/* cbs_mex.cpp
 *
 * Mex-dependant routines for cubic b-splines.
 * 
 * History
 *  2011/06/24: First version
 *
 * Note
 *  CBS is partly based on the original source code of BBS.
 *
 * (c)2011, Florent Brunet.
 */

/* This file is part of CBS.
 *
 * CBS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CBS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CBS.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "cbs_mex.h"

/* Check whether the mxArray "val" is a real scalar or not. "varname" is a
 string used to identified "val" in the error message. It can be NULL. */
void check_real_scalar(const mxArray *val, char *varname) {
    char str[128];
    if (!mxIsDouble(val) || mxIsComplex(val) || mxGetN(val)*mxGetM(val) != 1) {
        if (varname != NULL) {
            sprintf(str, "Value %s must be a real scalar.", varname);
            mexErrMsgTxt(str);
        } else
            mexErrMsgTxt("Value must be a real scalar.");
    }
}

/* Check if "val" is pure real, not complex (matrix or scalar). */
void check_real(const mxArray *val, char *varname) {
    char str[128];
    if (!mxIsDouble(val) || mxIsComplex(val))
        if (varname != NULL) {
            sprintf(str, "Value %s must be a real scalar.", varname);
            mexErrMsgTxt(str);
        } else
            mexErrMsgTxt("Value must be a real scalar.");
}

/* Check if the matrix of control points has the correct size, given the general parameters of a spline. */
void check_cbs_ctrlpts_size(const mxArray *val, cbs_t *cbs) {
    char errstr[128];
    if (mxGetM(val) != cbs->valdim || mxGetN(val) != cbs->npts) {
        sprintf(errstr, "The control points matrix must be of size (%d x %d).", cbs->valdim, cbs->npts);
        mexErrMsgTxt(errstr);
    }
}

/* Private function used by "array_to_cbs" to extract one field of type double
 from a cbs structure. The "structureness" of "val" is normally tested in
 "array_to_cbs". */
double get_cbs_field_double(const mxArray *val, char *field) {
    mxArray *a_field = NULL;
    char str[96];
    
    a_field = mxGetField(val, 0, field);
    
    if (a_field) {
        check_real_scalar(a_field, field);
        return mxGetScalar(a_field);
    } else {
        sprintf(str, "'%s': non-existant field.", field);
        mexErrMsgTxt(str);
    }      
}

/* Same as "get_cbs_field_double" but for values of type "int". */
int get_cbs_field_int(const mxArray *val, char *field) {
    mxArray *a_field = NULL;
    char str[96];
    
    a_field = mxGetField(val, 0, field);
    
    if (a_field) {
        check_real_scalar(a_field, field);
        return (int)mxGetScalar(a_field);
    } else {
        sprintf(str, "'%s': non-existant field.", field);
        mexErrMsgTxt(str);
    }      
}

/* Convert a Matlab structure (an mxArray) to a "cbs_t" structure.
   This function also checks for the validity of the matlab structure "val". */
void array_to_cbs(const mxArray *val, cbs_t *cbs) {   
    if (!mxIsStruct(val))
        mexErrMsgTxt("The spline structure is not a structure.");
    
    if (mxGetNumberOfElements(val) != 1)
        mexErrMsgTxt("The spline structure must not be an array of structures.");
    
    if (mxGetNumberOfFields(val) != 4)
        mexErrMsgTxt("The spline structure must contain 4 fields.");
    
    cbs->tmin = get_cbs_field_double(val, "tmin");
    cbs->tmax = get_cbs_field_double(val, "tmax");
    cbs->npts = get_cbs_field_int(val, "npts");
    
    cbs->valdim = get_cbs_field_int(val, "valdim");
}
